@@ -1,12 +1,10 @@
-use std::sync::Arc;
-
 use google_cloud_googleapis::{Code, Status};
 use tonic::transport::Channel;
-use tonic::{IntoRequest, Request, Response, Streaming};
+use tonic::{Response, Streaming};
 
-use google_cloud_auth::token_source::TokenSource;
 use google_cloud_gax::call_option::{Backoff, BackoffRetrySettings, BackoffRetryer};
 use google_cloud_gax::invoke::invoke_reuse;
+use google_cloud_gax::util::create_request;
 use google_cloud_googleapis::spanner::v1 as internal;
 use google_cloud_googleapis::spanner::v1::spanner_client::SpannerClient;
 use google_cloud_googleapis::spanner::v1::{
@@ -17,6 +15,7 @@ use google_cloud_googleapis::spanner::v1::{
     PartitionReadRequest, PartitionResponse, ReadRequest, ResultSet, RollbackRequest, Session,
     Transaction,
 };
+use google_cloud_grpc::conn::TokenSource;
 
 pub(crate) fn ping_query_request(session_name: impl Into<String>) -> internal::ExecuteSqlRequest {
     internal::ExecuteSqlRequest {
@@ -47,15 +46,12 @@ fn default_setting() -> BackoffRetrySettings {
 #[derive(Clone)]
 pub struct Client {
     inner: SpannerClient<Channel>,
-    token_source: Option<Arc<dyn TokenSource>>,
+    token_source: TokenSource,
 }
 
 impl Client {
     /// create new spanner client
-    pub fn new(
-        inner: SpannerClient<Channel>,
-        token_source: Option<Arc<dyn TokenSource>>,
-    ) -> Client {
+    pub fn new(inner: SpannerClient<Channel>, token_source: TokenSource) -> Client {
         Client {
             inner,
             token_source,
@@ -96,7 +92,7 @@ impl Client {
     ) -> Result<Response<Session>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let database = &req.database;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("database={}", database), &token, req.clone());
@@ -122,7 +118,7 @@ impl Client {
     ) -> Result<tonic::Response<BatchCreateSessionsResponse>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let database = &req.database;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("database={}", database), &token, req.clone());
@@ -147,7 +143,7 @@ impl Client {
     ) -> Result<Response<Session>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let name = &req.name;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("name={}", name), &token, req.clone());
@@ -170,7 +166,7 @@ impl Client {
     ) -> Result<Response<ListSessionsResponse>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let database = &req.database;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("database={}", database), &token, req.clone());
@@ -195,7 +191,7 @@ impl Client {
     ) -> Result<Response<()>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let name = &req.name;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("name={}", name), &token, req.clone());
@@ -228,7 +224,7 @@ impl Client {
     ) -> Result<Response<ResultSet>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -255,7 +251,7 @@ impl Client {
     ) -> Result<Response<Streaming<PartialResultSet>>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -288,7 +284,7 @@ impl Client {
     ) -> Result<Response<ExecuteBatchDmlResponse>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -337,7 +333,7 @@ impl Client {
     ) -> Result<Response<ResultSet>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -364,7 +360,7 @@ impl Client {
     ) -> Result<Response<Streaming<PartialResultSet>>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -390,7 +386,7 @@ impl Client {
     ) -> Result<Response<Transaction>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -426,7 +422,7 @@ impl Client {
     ) -> Result<Response<CommitResponse>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -456,7 +452,7 @@ impl Client {
     ) -> Result<Response<()>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -489,7 +485,7 @@ impl Client {
     ) -> Result<Response<PartitionResponse>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -524,7 +520,7 @@ impl Client {
     ) -> Result<Response<PartitionResponse>, Status> {
         let mut setting = Client::get_call_setting(opt);
         let session = &req.session;
-        let token = self.get_token().await?;
+        let token = self.token_source.token().await?;
         return invoke_reuse(
             |spanner_client| async {
                 let request = create_request(format!("session={}", session), &token, req.clone());
@@ -538,34 +534,4 @@ impl Client {
         )
         .await;
     }
-
-    async fn get_token(&self) -> Result<Option<String>, Status> {
-        match &self.token_source {
-            Some(token_source) => token_source
-                .token()
-                .await
-                .map_err(|e| {
-                    Status::new(tonic::Status::new(
-                        tonic::Code::Unauthenticated,
-                        format!("token error: {:?}", e),
-                    ))
-                })
-                .map(|v| Some(v.value())),
-            None => Ok(None),
-        }
-    }
-}
-
-fn create_request<T>(
-    param_string: String,
-    token: &Option<String>,
-    into_request: impl IntoRequest<T>,
-) -> Request<T> {
-    let mut request = into_request.into_request();
-    let target = request.metadata_mut();
-    target.append("x-goog-request-params", param_string.parse().unwrap());
-    if token.is_some() {
-        target.insert("authorization", token.as_ref().unwrap().parse().unwrap());
-    };
-    request
 }
